SystemOrganization addCategory: #'Idiom-Exercise'!

TestCase subclass: #IdiomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!IdiomTest methodsFor: 'testing' stamp: 'JuanIgnacio 4/23/2013 19:34'!
should: aBlock runInLessThan: timeLimitInMilliseconds
	"comment stating purpose of message"

	| millisecondsBeforeRunning millisecondsAfterRunning |
	millisecondsBeforeRunning := Time millisecondClockValue.
	aBlock value.
	millisecondsAfterRunning := Time millisecondClockValue.
	
	self assert: (millisecondsAfterRunning-millisecondsBeforeRunning) < timeLimitInMilliseconds ! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanIgnacio 4/23/2013 19:36'!
should: aBlock throw: anException with: anExceptionHandler

	[ aBlock value.	self fail ]
		on: anException 
		do: anExceptionHandler.! !

!IdiomTest methodsFor: 'testing' stamp: 'HernanWilkinson 5/9/2012 18:15'!
test01AddingCustomerShouldNotTakeMoreThan50Milliseconds

	| customerBook millisecondsBeforeRunning millisecondsAfterRunning|
	
	customerBook := CustomerBook  new.
	
	millisecondsBeforeRunning := Time millisecondClockValue.
	customerBook addCustomerNamed: 'John Lennon'.
	millisecondsAfterRunning := Time millisecondClockValue.
	
	self assert: (millisecondsAfterRunning-millisecondsBeforeRunning) < 50 
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanIgnacio 4/23/2013 19:33'!
test02RemovingCustomerShouldNotTakeMoreThan100Milliseconds

	| customerBook johnLennon |
	
	customerBook := CustomerBook  new.
	johnLennon := 'John Lennon'.
	
	customerBook addCustomerNamed: johnLennon.
	 
	self
	should: [ customerBook removeCustomerNamed: johnLennon.] 
	runInLessThan: 100.
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'HernanWilkinson 5/9/2012 18:12'!
test03CanNotAddACustomerWithEmptyName 

	| customerBook |
			
	customerBook := CustomerBook new.

	[ customerBook addCustomerNamed: ''.
	self fail ]
		on: Error 
		do: [ :anError | 
			self assert: anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage.
			self assert: customerBook isEmpty ]! !

!IdiomTest methodsFor: 'testing' stamp: 'HernanWilkinson 5/16/2012 16:50'!
test04CanNotRemoveAndInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	[ customerBook removeCustomerNamed: 'Paul McCartney'.
	self fail ]
		on: NotFound 
		do: [ :anError | 
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: johnLennon) ]
! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanIgnacio 4/23/2013 19:34'!
test05AddingCustomerShouldNotTakeMoreThan50Milliseconds

	| customerBook |
	
	customerBook := CustomerBook  new.
	
	self 
		should: [ customerBook addCustomerNamed: 'John Lennon'. ]
		runInLessThan: 50.
	
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanIgnacio 4/23/2013 19:34'!
test06RemovingCustomerShouldNotTakeMoreThan100Milliseconds

	| customerBook johnLennon |
	
	customerBook := CustomerBook  new.
	johnLennon := 'John Lennon'.
	
	customerBook addCustomerNamed: johnLennon.
	 
	self
	should: [ customerBook removeCustomerNamed: johnLennon.] 
	runInLessThan: 100.
	
! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanIgnacio 4/23/2013 19:37'!
test07CanNotAddACustomerWithEmptyName 

	| customerBook |
			
	customerBook := CustomerBook new.

	self 
		should: [ customerBook addCustomerNamed: ''. ]
		throw: Error 
		with: [ :anError | 
			self assert: anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage.
			self assert: customerBook isEmpty ]! !

!IdiomTest methodsFor: 'testing' stamp: 'JuanIgnacio 4/23/2013 19:40'!
test08CanNotRemoveAndInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	self 
		should:	[ customerBook removeCustomerNamed: 'Paul McCartney'. ]
		throw: NotFound 
		with: [ :anError | 
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: johnLennon) ]
! !

Object subclass: #CustomerBook
	instanceVariableNames: 'customers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Idiom-Exercise'!

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:57'!
customerAlreadyExistsErrorMessage

	^'Customer already exists'! !

!CustomerBook class methodsFor: 'error messages' stamp: 'HernanWilkinson 7/6/2011 17:53'!
customerCanNotBeEmptyErrorMessage

	^'Customer name cannot be empty'! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:42'!
addCustomerNamed: aName

	aName isEmpty ifTrue: [ self signalCustomerNameCannotBeEmpty ].
	(customers includes: aName) ifTrue: [ self signalCustomerAlreadyExists ].
	
	customers add: aName ! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:56'!
includesCustomerNamed: aName

	^customers includes: aName ! !

!CustomerBook methodsFor: 'initialization' stamp: 'HernanWilkinson 7/6/2011 17:42'!
initialize

	super initialize.
	customers := OrderedCollection new! !

!CustomerBook methodsFor: 'testing' stamp: 'HernanWilkinson 7/6/2011 17:48'!
isEmpty
	
	^customers isEmpty  ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:56'!
numberOfCustomers
	
	^customers size! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:44'!
removeCustomerNamed: aName
 
	customers remove: aName ! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:52'!
signalCustomerAlreadyExists 

	self error: self class customerAlreadyExistsErrorMessage! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:51'!
signalCustomerNameCannotBeEmpty 

	self error: self class customerCanNotBeEmptyErrorMessage ! !
